unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

const
  AM_GOTDATA = WM_USER + $99;
type
  TMainForm = class(TForm)
    Panel1: TPanel;
    EndButton: TButton;
    LogList: TListBox;
    MainPaint: TPaintBox;
    Splitter1: TSplitter;
    BlockLabel: TLabel;
    SetConfigButton: TButton;
    Drop1FButton: TButton;
    Raise1FButton: TButton;
    FirmCalButton: TButton;
    Cal1TO1Button: TButton;
    CalULoBHiButton: TButton;
    WriteFFButton: TButton;
    CalAUTOButton: TButton;
    AddrDataTestButton: TButton;
    AbortClearButton: TButton;
    GPIFContButton: TButton;
    GPIFTCEButton: TButton;
    ClockGoButton: TButton;
    StartButton: TButton;
    StopClockButton: TButton;
    StartClockButton: TButton;
    LogLabel: TLabel;
    RangeRadios: TRadioGroup;
    ScanCheck: TCheckBox;
    editRate: TEdit;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure EndButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure SetConfigButtonClick(Sender: TObject);
    procedure Drop1FButtonClick(Sender: TObject);
    procedure Raise1FButtonClick(Sender: TObject);
    procedure FirmCalButtonClick(Sender: TObject);
    procedure WriteFFButtonClick(Sender: TObject);
    procedure Cal1TO1ButtonClick(Sender: TObject);
    procedure CalULoBHiButtonClick(Sender: TObject);
    procedure GPIFContButtonClick(Sender: TObject);
    procedure GPIFTCEButtonClick(Sender: TObject);
    procedure CalAUTOButtonClick(Sender: TObject);
    procedure ClockGoButtonClick(Sender: TObject);
    procedure AbortClearButtonClick(Sender: TObject);
    procedure AddrDataTestButtonClick(Sender: TObject);
    procedure StopClockButtonClick(Sender: TObject);
    procedure StartClockButtonClick(Sender: TObject);
  protected
    procedure AMGotData(var Msg: TMessage); message AM_GOTDATA;
    function LogLine(Lin: String): Integer;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math, ScanBitmaps, AIOUSB;

type
  PContCallback = procedure (pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl;

function ADC_BulkMode(DeviceIndex, BulkMode: LongWord): LongWord; cdecl; external 'AIOUSB.dll';
function ADC_BulkContinuousCallbackStart(DeviceIndex: LongWord; BufSize: LongWord; BaseBufCount: LongWord; Context: LongWord; pCallback: Pointer): LongWord; cdecl; external 'AIOUSB.dll';
function ADC_BulkContinuousEnd(DeviceIndex: LongWord; pIOStatus: PLongWord): LongWord; cdecl; external 'AIOUSB.dll';

const
  VGAColor: array[$0..$F] of TColor = ($404040, $000080, $008000, $008080, $800000, $800080, $808000, $C0C0C0, $808080, $0000FF, $00FF00, $00FFFF, $FF0000, $FF00FF, $FFFF00, $FFFFFF);
  DeviceIndex = 0;

var
  Config: array of Byte;
  ConfigSize: LongWord;
  hBlockMutex: THandle;
  BlockID, LastBlockID: LongWord;
  Block: array of Word;
  BlockUsed: Integer;

  HF: Single;
  BackBuffer: TScanBitmap;
  bNeedsResize: Boolean;
  //Hist: array[Word] of Boolean;
  //HistClock: Byte;
  RawLog: array[Word] of Word;
  RawNext: Integer;
  RawState: Integer;



function IntToBin(I, MinBits: LongWord): String;
var
  DoneBits: LongWord;

begin
  Result := '';
  DoneBits := 0;
  repeat
    Result := Char(Byte('0') + (I and 1)) + Result;
    I := I shr 1;
    Inc(DoneBits);
  until (I = 0) and (DoneBits >= MinBits);
end;

procedure DumpStringToFile(S, Path: String);
var
  Fil: TFileStream;

begin
  Fil := TFileStream.Create(Path, fmCreate or fmShareDenyWrite);
  try
    Fil.Write(S[1], Length(S));
  finally
    Fil.Free;
  end;
end;

procedure MyReadData(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl;
var
  L: Integer;

begin
  WaitForSingleObject(hBlockMutex, INFINITE);
    L := BufSize div 2;
    if L > Length(Block) then SetLength(Block, L);
    Move(pBuf^, Block[0], 2*L);
    BlockUsed := L;

    Inc(BlockID);
  ReleaseMutex(hBlockMutex);
  PostMessage(MainForm.Handle, AM_GOTDATA, 0, 0);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  I: Integer;
  
begin
  hBlockMutex := CreateMutex(nil, False, nil);
  BackBuffer := TScanBitmap.Create;
  BackBuffer.PixelFormat := pf32bit;
  //BackBuffer.Width := 223;
  //BackBuffer.Height := 146;
  BackBuffer.Canvas.Brush.Color := $000000;
  bNeedsResize := True;

  if GetDevices = 0 then begin
    LogLine('No devices found.');
  end
  else begin
    ConfigSize := 21;
    SetLength(Config, ConfigSize);
    for I := 0 to ConfigSize - 1 do Config[I] := 0;
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  CloseHandle(hBlockMutex);
  BackBuffer.Free;
end;

procedure TMainForm.StartButtonClick(Sender: TObject);
begin
  LogLine('Start sequence...');

  RawNext := 0;
  RawState := 0;

  GPIFContButton.Click;

  SetConfigButton.Click;

  //if False then begin
    //Drop1FButton.Click;

    //GPIFContButton.Click;

    //WriteFFButton.Click;

    //Cal1TO1Button.Click;

    //CalULoBHiButton.Click;
  //end;

  ClockGoButton.Click;
  LogLine('Started.');
end;

procedure TMainForm.EndButtonClick(Sender: TObject);
var
  IOStatus: LongWord;
  I: Integer;
  Buf: String;

begin
  LogLine('Ending...');
  ADC_BulkContinuousEnd(DeviceIndex, @IOStatus);
  if IOStatus = ERROR_SUCCESS then
    LogLine('Ended.')
  else
    LogLine('Ended with I/O error ' + IntToStr(IOStatus) + '.')
  ;

  if RawState = 2 then begin
    Buf := '';
    for I := 0 to $FFFF do begin
      Buf := Buf + IntToBin(RawLog[I], 16);
      if (I mod 3) <> 2 then Buf := Buf + #9 else Buf := Buf + #13#10;
    end;
    DumpStringToFile(Buf, 'Log.txt');
    LogLine('Log Dumped (Your Fault)');
  end;

  StopClockButton.Click;
end;

procedure TMainForm.AMGotData(var Msg: TMessage);
var
  R: TRect;
  X, Y, I: Integer;
  ThisBlockID: LongWord;
  bNewBlock, bDraw: Boolean;
  Buf: String;

begin
  Msg.Result := 0;

  if bNeedsResize then begin
    R := MainPaint.ClientRect;
    BackBuffer.Height := 0;
    BackBuffer.Width := R.Right;
    BackBuffer.Height := R.Bottom;
    BackBuffer.ClearTable;
    HF := (BackBuffer.Height - 1) / $FFFF;
  end
  else begin
    R.Left := 0;
    R.Top := 0;
    R.Right := BackBuffer.Width;
    R.Bottom := BackBuffer.Height;
    BackBuffer.Canvas.FillRect(R);
  end;

  bDraw := False;
  WaitForSingleObject(hBlockMutex, INFINITE);
    ThisBlockID := BlockID;
    bNewBlock := ThisBlockID <> LastBlockID;
    if bNewBlock then try
      LastBlockID := ThisBlockID;
      bDraw := BlockUsed <> 0;
      if bDraw then begin
        for X := 0 to Min(BlockUsed, R.Right) - 1 do begin
          Y := Round(HF * not Block[X]);
          I := X and $F;
          BackBuffer.ScanPixel[X, Y] := VGAColor[I];
          //if I = 3 then Hist[Block[X]] := True;
          {
          if (RawState = 0) and (I >= 2) and (I <= 4) then begin
            RawLog[RawNext] := Block[X];
            Inc(RawNext);
            if RawNext >= Length(RawLog) then RawState := 1;
          end;
          }
        end;
        if (LastBlockID and $F) = 0 then begin
          Buf := '';
          for X := 0 to Min(BlockUsed, 16) - 1 do
            Buf := Buf + (IntToHex(Block[X], 4) + #13#10)
          ;
          LogLabel.Caption := Buf;
        end;
      end;
    except
    end;
  ReleaseMutex(hBlockMutex);

  if bNewBlock then BlockLabel.Caption := IntToStr(ThisBlockID);
  if bDraw then begin
    BitBlt(MainPaint.Canvas.Handle, 0, 0, BackBuffer.Width, BackBuffer.Height, BackBuffer.Canvas.Handle, 0, 0, SRCCOPY);
    if RawState = 1 then begin
      RawState := 2;
      LogLine('Log Ready');
    end;
    {
    Inc(HistClock);
    if HistClock = 0 then begin
      Buf := '';
      for I := 0 to $FFFF do if Hist[I] then Buf := Buf + (IntToBin(I, 16) + #13#10);
      HistLabel.Caption := Buf;
    end;
    }
  end;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  bNeedsResize := True;
end;

function TMainForm.LogLine(Lin: String): Integer;
begin
  Result := LogList.Items.Add(Lin);
  LogList.ItemIndex := Result;
end;

procedure TMainForm.SetConfigButtonClick(Sender: TObject);
var
  Status: LongWord;
  I: Integer;
  Buf: String;

begin
  LogLine('Configuring...');
  for I := 0 to ConfigSize - 1 do Config[I] := 0;

  {
  //for I := 0 to 15 do Config[I] := 1;
  for I := 0 to 7 do Config[2*I] := 1;
  //Config[4] := 1;
  //for I := 0 to 7 do Config[I] := 1;
  }
  case RangeRadios.ItemIndex of
    0, 1: for I := 0 to 15 do Config[I] := RangeRadios.ItemIndex;
    2: for I := 0 to 7 do Config[2*I] := 1;
    3: for I := 0 to 7 do Config[I] := 1;
  end;
  Buf := 'Ranges:';
  for I := 0 to 15 do Buf := Buf + (' ' + IntToHex(Config[I], 1));
  LogLine(Buf);

  if ScanCheck.Checked then Config[$11] := $15 else Config[$11] := $11;
  Config[$12] := $F0;    // BOIAN SPECIFIC: 4 CHANNELS TOTAL
  LogLine('Channel config: ' +IntToHex(Config[$12],2));
  Status := ADC_SetConfig(DeviceIndex, @Config[0], ConfigSize);
  if Status = ERROR_SUCCESS then
    LogLine('Configured.')
  else
    LogLine('Error ' + IntToStr(Status) + ' from ADC_SetConfig.')
  ;
end;

procedure TMainForm.Drop1FButtonClick(Sender: TObject);
var
  D4Data: Byte;

begin
  LogLine('Writing 00 to 1F...');
  D4Data := $00;
  GenericVendorWrite(DeviceIndex, $D4, 0, $1F, SizeOf(D4Data), @D4Data);
  LogLine('Wrote.');
end;

procedure TMainForm.Raise1FButtonClick(Sender: TObject);
var
  D4Data: Byte;

begin
  LogLine('Writing 02 to 1F...');
  D4Data := $02;
  GenericVendorWrite(DeviceIndex, $D4, 0, $1F, SizeOf(D4Data), @D4Data);
  LogLine('Wrote.');
end;

procedure TMainForm.FirmCalButtonClick(Sender: TObject);
var
  D4Data: Byte;

begin
  LogLine('Calibrating 1:1 via firmware...');
  D4Data := $00;
  GenericVendorWrite(DeviceIndex, $D4, 0, $FC, SizeOf(D4Data), @D4Data);
  LogLine('Calibrated.');
end;

procedure TMainForm.WriteFFButtonClick(Sender: TObject);
var
  D4Data: Byte;

begin
  LogLine('Writing 00 to FF...');
  D4Data := $00;
  GenericVendorWrite(DeviceIndex, $D4, 0, $FF, SizeOf(D4Data), @D4Data);
  LogLine('Wrote.');
end;

procedure TMainForm.Cal1TO1ButtonClick(Sender: TObject);
begin
  LogLine('Calibrating...');
  ADC_SetCal(DeviceIndex, ':1TO1:');
  LogLine('Calibrated.');
end;

procedure TMainForm.CalAUTOButtonClick(Sender: TObject);
begin
  LogLine('Calibrating...');
  ADC_SetCal(DeviceIndex, ':AUTO:');
  LogLine('Calibrated.');
end;

procedure TMainForm.CalULoBHiButtonClick(Sender: TObject);
var
  Status: LongWord;

begin
  LogLine('Calibrating...');
  Config[$10] := $01; //Select unip low ref, to select unip cal bank(on banked boards).
  ADC_SetConfig(DeviceIndex, @Config[0], ConfigSize);
  Status := ADC_SetCal(DeviceIndex, 'T:\LowCal.bin');
  if Status <> ERROR_SUCCESS then LogLine('Error ' + IntToStr(Status) + ' on unipolar cal.');

  Config[$10] := $05; //Select bip low ref, to select bip cal bank(on banked boards).
  ADC_SetConfig(DeviceIndex, @Config[0], ConfigSize);
  Status := ADC_SetCal(DeviceIndex, 'T:\HighCal.bin');
  if Status <> ERROR_SUCCESS then LogLine('Error ' + IntToStr(Status) + ' on bipolar cal.');

  Config[$10] := 0; //Put cal config back to taking real data.
  ADC_SetConfig(DeviceIndex, @Config[0], ConfigSize);
  LogLine('Calibrated.');
end;

procedure TMainForm.GPIFContButtonClick(Sender: TObject);
//var
//  D4Data: Byte;

begin
  LogLine('Setting GPIF to continuous...');
  //D4Data := $00;
  //GenericVendorWrite(DeviceIndex, $D4, 0, $FE, SizeOf(D4Data), @D4Data);
  ADC_BulkMode(DeviceIndex, $1);
  LogLine('Set GPIF.');
end;

procedure TMainForm.GPIFTCEButtonClick(Sender: TObject);
//var
//  D4Data: Byte;

begin
  LogLine('Setting GPIF to block...');
  //D4Data := $00;
  //GenericVendorWrite(DeviceIndex, $D4, 0, $FD, SizeOf(D4Data), @D4Data);
  ADC_BulkMode(DeviceIndex, $0);
  LogLine('Set GPIF.');
end;

procedure TMainForm.ClockGoButtonClick(Sender: TObject);
var
  Status: LongWord;

begin
  StartClockButton.Click;

  LogLine('Starting...');

  Status := ADC_BulkContinuousCallbackStart(DeviceIndex, 10240, 128, 0, @MyReadData);
  if Status = ERROR_SUCCESS then
    LogLine('OK!')
  else
    LogLine('Error ' + IntToStr(Status) + ' from ADC_BulkContinuousCallbackStart.')
  ;
end;

procedure TMainForm.AbortClearButtonClick(Sender: TObject);
begin
  LogLine('Clearing FIFO(and aborting stream)...');
  AIOUSB_ClearFIFO(DeviceIndex, 5);
  LogLine('Cleared.');
end;

procedure TMainForm.AddrDataTestButtonClick(Sender: TObject);
const
  REQ_ADDRDATALINE_TEST = $91;

var
  DataSize, Status: LongWord;
  ReturnCode: Byte;
  ReturnStr: String;

begin
  LogLine('Checking test result...');

  DataSize := 1;
  ReturnCode := $FF;
  Status := GenericVendorRead(diOnly, REQ_ADDRDATALINE_TEST, 0, 0, DataSize, @ReturnCode);
  if Status <> ERROR_SUCCESS then begin
    LogLine('Error ' + IntToStr(Status) + ' in req 91h.');
    Exit;
  end;
  case ReturnCode of
    $00: ReturnStr := 'Pass';
    $01: ReturnStr := 'Hardware Failure';
    $FE: ReturnStr := 'Test Skipped';
    else ReturnStr := 'Firmware Failure #' + IntToHex(ReturnCode, 2);
  end;

  LogLine('Addr/data line test: ' + ReturnStr + '.');
end;

procedure TMainForm.StopClockButtonClick(Sender: TObject);
var
  Status: LongWord;
  Hz: Double;

begin
  Hz := 0;
  LogLine('Clock 0...');
  Status := CTR_StartOutputFreq(DeviceIndex, 0, @Hz);
  if Status = ERROR_SUCCESS then
    LogLine('Clock set.')
  else
    LogLine('Error ' + IntToStr(Status) + ' setting clock.')
  ;
end;

procedure TMainForm.StartClockButtonClick(Sender: TObject);
var
  Status: LongWord;
  Hz: Double;

begin
  Hz := StrToFloat(editRate.Text);
  LogLine('Clock ' + FloatToStr(Hz) + 'Hz...');
  Status := CTR_StartOutputFreq(DeviceIndex, 0, @Hz);
  if Status = ERROR_SUCCESS then
    LogLine('Clock set.')
  else
    LogLine('Error ' + IntToStr(Status) + ' setting clock.')
  ;
end;

end.
